/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2017 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <stdio.h>

#include "cmsis_os2.h"                  /* ::CMSIS:RTOS2 */
#include "Board_LED.h"                  /* ::Board Support:LED */
#include "Board_Buttons.h"              /* ::Board Support:Buttons */

#include "RTE_Components.h"             /* Component selection */
#include CMSIS_device_header

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

volatile int32_t delay_val = 500;

osThreadId_t tid_thrLED;                /* Thread id of thread: LED */
osThreadId_t tid_thrBUT;                /* Thread id of thread: BUT */

/*----------------------------------------------------------------------------
  thrLED: blink LED
 *----------------------------------------------------------------------------*/
void thrLED(void *argument) {
  uint32_t led_max    = LED_GetCount();
  uint32_t led_num    = 0;

  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_On(led_num);                                           /* Turn specified LED on */
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);
    LED_Off(led_num);                                          /* Turn specified LED off */

    led_num++;                                                 /* Change LED number */
    if (led_num >= led_max) {
      led_num = 0;                                             /* Restart with first LED */
    }
  }

}

/*----------------------------------------------------------------------------
  thrBUT: check button state
 *----------------------------------------------------------------------------*/
void thrBUT(void *argument) {
  uint32_t button_msk = (1U << Buttons_GetCount()) - 1;

  for (;;) {
    osDelay(delay_val);                                        /* Wait */
    while (Buttons_GetState() & (button_msk));                 /* Wait while holding USER button */
    osThreadFlagsSet(tid_thrLED, 0x0001);
  }

}

/*----------------------------------------------------------------------------
  configure SystemCoreClock:
 *----------------------------------------------------------------------------*/
void SystemCoreClockConfigure(void) {

	/* add code */
}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {
 
  tid_thrBUT = osThreadNew (thrBUT, NULL, NULL);               /* create BUT thread */
  tid_thrLED = osThreadNew (thrLED, NULL, NULL);               /* create LED thread */

  for (;;) {}
}
 
/*----------------------------------------------------------------------------
  main: create tasks and start kernel
 *----------------------------------------------------------------------------*/
int main (void) {
  uint32_t button_msk = (1U << Buttons_GetCount()) - 1;

  // System Initialization
  SystemCoreClockConfigure();
  SystemCoreClockUpdate();

  LED_Initialize();                                            /* initalize LEDs */
  Buttons_Initialize();                                        /* initalize Buttons */

#ifdef RTE_Compiler_EventRecorder
  EventRecorderInitialize(EventRecordAPI, 1U);                /* Initialize and start Event Recorder */
#endif

  osKernelInitialize ();                                       /* initialize CMSIS-RTOS */

  osThreadNew(app_main, NULL, NULL);                           /* create application main thread */

  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                                           /* start thread execution */
  }
}
